## This script performs regression analyses for each of 100 stochastic maps (input files are included in folder "stochastic_maps_independent_events"). These regressions are described in section (b) of the Material and methods. Note that only results from log-transformed richness are described in the manuscript, but raw results were congruent.

# Set working directory to access 100 stochastic maps files

setwd("~/Desktop/fish files may 2017/stochastic_results_sep15")

results_list <- list(NA)

pvalues_max <- as.data.frame(matrix(NA, 100, 4))
names(pvalues_max) <- c("p_raw", "r2_raw", "p_log", "r2_log")

pvalues_sum <- as.data.frame(matrix(NA, 100, 4))
names(pvalues_sum) <- c("p_raw", "r2_raw", "p_log", "r2_log")

pvalues_numb <- as.data.frame(matrix(NA, 100, 4))
names(pvalues_numb) <- c("p_raw", "r2_raw", "p_log", "r2_log")


areas <- c("A", "B", "C", "D", "E", "F", "G", "H", "I")

for(map in 1:100){
  print(map)
  
  map_data <- read.csv(paste0("independent_nodes+tips_stochastic_", map, ".csv"))
  
### IMPORTANT: for maps with fossil constraints, need to force colonizations along tips only to CIP to be 34 or less. This is because if a new colonization occured at a terminal taxa, we took half the stem age as the age of the colonization. However, some tips have a stem age >34 myr. 
### Turn this code off if not using fossil constraints.
#for(i in 1:length(d$D)){
#  if(is.na(d$D_ages[i])){}else{
#    if(d$D_ages[i]>34){
#      d$D_ages[i]<-34
#    }}}

  
  data <- matrix(areas,9,4)
  data[,2:4] <- 1
  data <- data.frame(data)
  colnames(data) <- c("area", "max_age", "num_col", "summed_age")
  data$max_age <- as.numeric(data$max_age)
  data$num_col <- as.numeric(data$num_col)
  data$summed_age <- as.numeric(data$summed_age)
  
  
  ### add max ages to data
  data[1,2] <- max(map_data$A_ages, na.rm=TRUE)
  data[2,2] <- max(map_data$B_ages, na.rm=TRUE)
  data[3,2] <- max(map_data$C_ages, na.rm=TRUE)
  data[4,2] <- max(map_data$D_ages, na.rm=TRUE)
  data[5,2] <- max(map_data$E_ages, na.rm=TRUE)
  data[6,2] <- max(map_data$F_ages, na.rm=TRUE)
  data[7,2] <- max(map_data$G_ages, na.rm=TRUE)
  data[8,2] <- max(map_data$H_ages, na.rm=TRUE)
  data[9,2] <- max(map_data$I_ages, na.rm=TRUE)
  
  ### add richness (in tree; log and raw) to data
  richness <- c(1845, 1210, 2677, 5659, 3697, 1570, 1749, 929, 6584) ### this is the regional richness in Table S2 in ESM
  
  data$richness <- richness
  data$log_richness <- log10(richness)
  
  ### add summed colonization ages to data
  data[1,4] <- sum(map_data$A_ages, na.rm=TRUE)
  data[2,4] <- sum(map_data$B_ages, na.rm=TRUE)
  data[3,4] <- sum(map_data$C_ages, na.rm=TRUE)
  data[4,4] <- sum(map_data$D_ages, na.rm=TRUE)
  data[5,4] <- sum(map_data$E_ages, na.rm=TRUE)
  data[6,4] <- sum(map_data$F_ages, na.rm=TRUE)
  data[7,4] <- sum(map_data$G_ages, na.rm=TRUE)
  data[8,4] <- sum(map_data$H_ages, na.rm=TRUE)
  data[9,4] <- sum(map_data$I_ages, na.rm=TRUE)
  
  
  ### add number colonizations to data
  
  data[1,3] <- length(which(!map_data$A_ages %in% NA))
  data[2,3] <- length(which(!map_data$B_ages %in% NA))
  data[3,3] <- length(which(!map_data$C_ages %in% NA))
  data[4,3] <- length(which(!map_data$D_ages %in% NA))
  data[5,3] <- length(which(!map_data$E_ages %in% NA))
  data[6,3] <- length(which(!map_data$F_ages %in% NA))
  data[7,3] <- length(which(!map_data$G_ages %in% NA))
  data[8,3] <- length(which(!map_data$H_ages %in% NA))
  data[9,3] <- length(which(!map_data$I_ages %in% NA))
  
  
  ## store values for later
  
  results_list[[map]] <- data
  
  #linear models
  
  ##### comparison 1: max age and richness
  
  #store p-value and r-squared in data frame for all 50 maps
  
  model1<- lm(richness~max_age, data=data) #raw values of richness
  
  model2<- lm(log_richness~max_age, data=data) #log values of richness
  
  pvalues_max[map, 1] <- summary(model1)$coefficients[8]
  pvalues_max[map, 2] <- summary(model1)$r.squared
  pvalues_max[map, 3] <- summary(model2)$coefficients[8]
  pvalues_max[map, 4] <- summary(model2)$r.squared
  
  
  ##### comparison 2: summed age and richness
  
  #store p-value and r-squared in data frame for all 50 maps
  
  model1<- lm(richness~summed_age, data=data) #raw values of richness
  
  model2<- lm(log_richness~summed_age, data=data) #log values of richness
  
  pvalues_sum[map, 1] <- summary(model1)$coefficients[8]
  pvalues_sum[map, 2] <- summary(model1)$r.squared
  pvalues_sum[map, 3] <- summary(model2)$coefficients[8]
  pvalues_sum[map, 4] <- summary(model2)$r.squared
  
  ##### comparison 3: number of colonizations and richness
  
  model1<- lm(richness~num_col, data=data) #raw values of richness
  
  model2<- lm(log_richness~num_col, data=data) #log values of richness
  
  pvalues_numb[map, 1] <- summary(model1)$coefficients[8]
  pvalues_numb[map, 2] <- summary(model1)$r.squared
  pvalues_numb[map, 3] <- summary(model2)$coefficients[8]
  pvalues_numb[map, 4] <- summary(model2)$r.squared
  
} #end loop through maps

### This won't be significant because it contains FW outlier

setwd("~/Desktop/fish files may 2017/allperc_alltimes_allareas")

write.csv(pvalues_max, file="allperc_alltimes_allareas_pvalues_max.csv")
write.csv(pvalues_sum, file="allperc_alltimes_allareas_pvalues_summed.csv")
write.csv(pvalues_numb, file="allperc_alltimes_allareas_pvalues_number.csv")


###################################################
##### Now remove freshwater outlier
###################################################

setwd("~/Desktop/fish files may 2017/stochastic_results_sep15")

results_list <- list(NA)

pvalues_max <- as.data.frame(matrix(NA, 100, 4))
names(pvalues_max) <- c("p_raw", "r2_raw", "p_log", "r2_log")

pvalues_sum <- as.data.frame(matrix(NA, 100, 4))
names(pvalues_sum) <- c("p_raw", "r2_raw", "p_log", "r2_log")

pvalues_numb <- as.data.frame(matrix(NA, 100, 4))
names(pvalues_numb) <- c("p_raw", "r2_raw", "p_log", "r2_log")


areas <- c("A", "B", "C", "D", "E", "F", "G", "H")

for(map in 1:100){
  print(map)
  
  map_data <- read.csv(paste0("independent_nodes+tips_stochastic_", map, ".csv"))
  
### IMPORTANT: for maps with fossil constraints, need to force colonizations along tips only to CIP to be 34 or less. This is because if a new colonization occured at a terminal taxa, we took half the stem age as the age of the colonization. However, some tips have a stem age >34 myr. 
### Turn this code off if not using fossil constraints.
#for(i in 1:length(d$D)){
#  if(is.na(d$D_ages[i])){}else{
#    if(d$D_ages[i]>34){
#      d$D_ages[i]<-34
#    }}}
  
  data <- matrix(areas,8,4)
  data[,2:4] <- 1
  data <- data.frame(data)
  colnames(data) <- c("area", "max_age", "num_col", "summed_age")
  data$max_age <- as.numeric(data$max_age)
  data$num_col <- as.numeric(data$num_col)
  data$summed_age <- as.numeric(data$summed_age)
  
  
  ### add max ages to data
  data[1,2] <- max(map_data$A_ages, na.rm=TRUE)
  data[2,2] <- max(map_data$B_ages, na.rm=TRUE)
  data[3,2] <- max(map_data$C_ages, na.rm=TRUE)
  data[4,2] <- max(map_data$D_ages, na.rm=TRUE)
  data[5,2] <- max(map_data$E_ages, na.rm=TRUE)
  data[6,2] <- max(map_data$F_ages, na.rm=TRUE)
  data[7,2] <- max(map_data$G_ages, na.rm=TRUE)
  data[8,2] <- max(map_data$H_ages, na.rm=TRUE)
  
  ### add richness (in tree; log and raw) to data
  richness <- c(1845, 1210, 2677, 5659, 3697, 1570, 1749, 929) ### this is the OBIS regional richness, will need to change for the circumstances
  data$richness <- richness
  data$log_richness <- log10(richness)
  
  ### add summed colonization ages to data
  data[1,4] <- sum(map_data$A_ages, na.rm=TRUE)
  data[2,4] <- sum(map_data$B_ages, na.rm=TRUE)
  data[3,4] <- sum(map_data$C_ages, na.rm=TRUE)
  data[4,4] <- sum(map_data$D_ages, na.rm=TRUE)
  data[5,4] <- sum(map_data$E_ages, na.rm=TRUE)
  data[6,4] <- sum(map_data$F_ages, na.rm=TRUE)
  data[7,4] <- sum(map_data$G_ages, na.rm=TRUE)
  data[8,4] <- sum(map_data$H_ages, na.rm=TRUE)
  
  
  ### add number colonizations to data
  
  data[1,3] <- length(which(!map_data$A_ages %in% NA))
  data[2,3] <- length(which(!map_data$B_ages %in% NA))
  data[3,3] <- length(which(!map_data$C_ages %in% NA))
  data[4,3] <- length(which(!map_data$D_ages %in% NA))
  data[5,3] <- length(which(!map_data$E_ages %in% NA))
  data[6,3] <- length(which(!map_data$F_ages %in% NA))
  data[7,3] <- length(which(!map_data$G_ages %in% NA))
  data[8,3] <- length(which(!map_data$H_ages %in% NA))
  
  
  ## store values for later
  
  results_list[[map]] <- data
  
  #linear models
  
  ##### comparison 1: max age and richness
  
  #store p-value and r-squared in data frame for all 50 maps
  
  model1<- lm(richness~max_age, data=data) #raw values of richness
  
  model2<- lm(log_richness~max_age, data=data) #log values of richness
  
  pvalues_max[map, 1] <- summary(model1)$coefficients[8]
  pvalues_max[map, 2] <- summary(model1)$r.squared
  pvalues_max[map, 3] <- summary(model2)$coefficients[8]
  pvalues_max[map, 4] <- summary(model2)$r.squared
  
  
  ##### comparison 2: summed age and richness
  
  #store p-value and r-squared in data frame for all 50 maps
  
  model1<- lm(richness~summed_age, data=data) #raw values of richness
  
  model2<- lm(log_richness~summed_age, data=data) #log values of richness
  
  pvalues_sum[map, 1] <- summary(model1)$coefficients[8]
  pvalues_sum[map, 2] <- summary(model1)$r.squared
  pvalues_sum[map, 3] <- summary(model2)$coefficients[8]
  pvalues_sum[map, 4] <- summary(model2)$r.squared
  
  ##### comparison 3: number of colonizations and richness
  
  model1<- lm(richness~num_col, data=data) #raw values of richness
  
  model2<- lm(log_richness~num_col, data=data) #log values of richness
  
  pvalues_numb[map, 1] <- summary(model1)$coefficients[8]
  pvalues_numb[map, 2] <- summary(model1)$r.squared
  pvalues_numb[map, 3] <- summary(model2)$coefficients[8]
  pvalues_numb[map, 4] <- summary(model2)$r.squared
  
} #end loop through maps

setwd("~/Desktop/fish files may 2017/allperc_alltimes_allareas_noFW")

write.csv(pvalues_max, file="allperc_alltimes_allareas_pvalues_max_noFW.csv")
write.csv(pvalues_sum, file="allperc_alltimes_allareas_pvalues_summed_noFW.csv")
write.csv(pvalues_numb, file="allperc_alltimes_allareas_pvalues_number_noFW.csv")


