# This script performs regression analyses on regional richness descended from events from 5.3 to 0 Ma, for all areas. 
# Scripts for analyses using combined regions are available from EC Miller by request

# Input files are in the folder "stochastic_maps_independent_events". Users will need to set their directory appropriately.

library(geiger)
library(phangorn)
tree <- read.tree("perc_tree_march2.tree")

####################
# What is needed to get time-for-speciation in a certain time bin:
# for each of 100 maps:
# summed ages
# number of cols
# richness of each area derived from events happening in that time period

#####################

### For 1 stochastic map
areas <- c("A", "B", "C", "D", "E", "F", "G", "H", "I")
test_list <- list(NA) #will fill data for each of 100 maps in here for analysis

for(map in 1:100){
  print(map)
  
d <- read.csv(paste0("independent_nodes+tips_stochastic_", map, ".csv"))
#max # is D

all <- read.csv(paste0(map, "_all_states.csv"))

### IMPORTANT: for maps with fossil constraints, need to force colonizations along tips only to CIP to be 34 or less. This is because if a new colonization occured at a terminal taxa, we took half the stem age as the age of the colonization. However, some tips have a stem age >34 myr. 
### Turn this code off if not using fossil constraints.
#for(i in 1:length(d$D)){
#  if(is.na(d$D_ages[i])){}else{
#    if(d$D_ages[i]>34){
#      d$D_ages[i]<-34
#    }}}


######### Set up a data set unique to this map, fill in values later

test <- matrix(areas,9,4)
test[,2:4] <- NA
test <- data.frame(test)
colnames(test) <- c("area", "richness", "num_col", "summed_age")
test$num_col <- as.numeric(test$num_col)
test$summed_age <- as.numeric(test$summed_age)
test$richness <- as.numeric(test$richness)


#################################################
########### Get the nodes for time period 5.3-0. 


#A

A_nodes_2 <- c()

for(i in 1:length(d$A_ages) ){
  if(is.na(d$A_ages[i])){}else{
  if(d$A_ages[i] <= 5.3 ){
    A_nodes_2 <- c(A_nodes_2, d$A[i])
  }}}


#B

B_nodes_2 <- c()

for(i in 1:length(d$B_ages) ){
  if(is.na(d$B_ages[i])){}else{
    if(d$B_ages[i] <= 5.3 ){
      B_nodes_2 <- c(B_nodes_2, d$B[i])
    }}}


#C

C_nodes_2 <- c()

for(i in 1:length(d$C_ages) ){
  if(is.na(d$C_ages[i])){}else{
    if(d$C_ages[i] <= 5.3 ){
      C_nodes_2 <- c(C_nodes_2, d$C[i])
    }}}


#D

D_nodes_2 <- c()

for(i in 1:length(d$D_ages) ){
  if(is.na(d$D_ages[i])){}else{
    if(d$D_ages[i] <= 5.3 ){
      D_nodes_2 <- c(D_nodes_2, d$D[i])
    }}}


#E

E_nodes_2 <- c()

for(i in 1:length(d$E_ages) ){
  if(is.na(d$E_ages[i])){}else{
    if(d$E_ages[i] <= 5.3 ){
      E_nodes_2 <- c(E_nodes_2, d$E[i])
    }}}


#F

F_nodes_2 <- c()

for(i in 1:length(d$F_ages) ){
  if(is.na(d$F_ages[i])){}else{
    if(d$F_ages[i] <= 5.3 ){
      F_nodes_2 <- c(F_nodes_2, d$F[i])
    }}}

#G

G_nodes_2 <- c()

for(i in 1:length(d$G_ages) ){
  if(is.na(d$G_ages[i])){}else{
    if(d$G_ages[i] <= 5.3 ){
      G_nodes_2 <- c(G_nodes_2, d$G[i])
    }}}

#H

H_nodes_2 <- c()

for(i in 1:length(d$H_ages) ){
  if(is.na(d$H_ages[i])){}else{
    if(d$H_ages[i] <= 5.3 ){
      H_nodes_2 <- c(H_nodes_2, d$H[i])
    }}}

#I

I_nodes_2 <- c()

for(i in 1:length(d$I_ages) ){
  if(is.na(d$I_ages[i])){}else{
    if(d$I_ages[i] <= 5.3 ){
      I_nodes_2 <- c(I_nodes_2, d$I[i])
    }}}



####################################################

### For each node, need to count descendants from that same colonization event. Exclude "secondary" events (i.e. transitions out of, and then back to, the same region).

################################################################################# state A

tip_rich <- length(which(A_nodes_2 < 4572)) #tip richness for D
clade_rich<- c()

## Now cycle thru the nodes

cycle <- A_nodes_2[which(A_nodes_2 >= 4572)]

for(i in 1:length(cycle)){
  clade <- extract.clade(tree, cycle[i])
  
  des <- Descendants(tree, cycle[i], type='all')
  
  nodes <- des[ which(des > length(tree$tip.label)) ] 
  
  tips <- des[ which(des < length(tree$tip.label)) ] #is equal to all tips descended from that node. So the node list is only useful for deleting clades belonging to other events.
  
  ### Check if any of the nodes are found in other events

  if(any(nodes %in% A_nodes_2)){
    x <- nodes[which(nodes %in% A_nodes_2)]
    for(y in x){
      desc <- unlist(Descendants(tree, y))
      tips <- tips[-which(tips %in% desc)]
    }
  }
  
  ### Check if any of the tips are found in other events

  if(any(tips %in% A_nodes_2)){
    tips <- tips[-which(tips %in% A_nodes_2) ]
  }
  
  #only get the tips with the correct state (D)
  tips <- tips[which(grepl( "A", all$states[tips]))]
  
  clade_rich<- c(clade_rich, length(tips)) #check that all tips have the same state included, or else there is an error
} #end loop, cycle through next node in D_list_1 (vector 'cycle')

#now clade_rich and tip_rich contain all richness derived from each independent colonization between the time period considered (here, 5.3-34 Ma)

A_rich <- sum(tip_rich + sum(clade_rich)) #richness of D derived from events occuring between 34-5.3 Ma

##### Some events don't actually lead to richness in the tree. Should subtract these nodes from sum and number counts.

cycle <- cycle[-which(clade_rich == 0)]
A_node_ages<- d[ which(d$A %in% cycle), 4 ] 
A_tip_ages <- d[ which(d$A %in% A_nodes_2[which(A_nodes_2<4572)]) , 4]
### Now we have all the information to fill in to 'data' for state D.

test[ 1,2 ]  <- A_rich
test[ 1,3 ] <- length(cycle)+tip_rich
test[ 1,4 ] <- sum(A_node_ages, A_tip_ages) 


################################################################################# state B

tip_rich <- length(which(B_nodes_2 < 4572)) #tip richness for B
clade_rich<- c()

## Now cycle thru the nodes

cycle <- B_nodes_2[which(B_nodes_2 >= 4572)]

for(i in 1:length(cycle)){
  clade <- extract.clade(tree, cycle[i])
  
  des <- Descendants(tree, cycle[i], type='all')
  
  nodes <- des[ which(des > length(tree$tip.label)) ] 
  
  tips <- des[ which(des < length(tree$tip.label)) ] #is equal to all tips descended from that node. So the node list is only useful for deleting clades belonging to other events.
  
  ### Check if any of the nodes are found in other events
  
  if(any(nodes %in% B_nodes_2)){
    x <- nodes[which(nodes %in% B_nodes_2)]
    for(y in x){
      desc <- unlist(Descendants(tree, y))
      tips <- tips[-which(tips %in% desc)]
    }
  }
  
  ### Check if any of the tips are found in other events
  
  if(any(tips %in% B_nodes_2)){
    tips <- tips[-which(tips %in% B_nodes_2) ]
  }
  
  #only get the tips with the correct state (D)
  tips <- tips[which(grepl( "B", all$states[tips]))]
  
  clade_rich<- c(clade_rich, length(tips)) #check that all tips have the same state included, or else there is an error
} #end loop, cycle through next node in D_list_1 (vector 'cycle')

#now clade_rich and tip_rich contain all richness derived from each independent colonization between the time period considered (here, 34-5.3 Ma)

B_rich <- sum(tip_rich + sum(clade_rich)) #richness of D derived from events occuring between 34-5.3 Ma

##### Some events don't actually lead to richness in the tree. Should subtract these nodes from sum and number counts.

cycle <- cycle[-which(clade_rich == 0)]
B_node_ages<- d[ which(d$B %in% cycle), 6 ] 
B_tip_ages <- d[ which(d$B %in% B_nodes_2[which(B_nodes_2<4572)]) , 6]
### Now we have all the information to fill in to 'data' for state D.

test[ 2,2 ]  <- B_rich
test[ 2,3 ] <- length(cycle)+tip_rich
test[ 2,4 ] <- sum(B_node_ages, B_tip_ages) 



################################################################################# state C

tip_rich <- length(which(C_nodes_2 < 4572)) #tip richness for B
clade_rich<- c()

## Now cycle thru the nodes

cycle <- C_nodes_2[which(C_nodes_2 >= 4572)]

for(i in 1:length(cycle)){
  clade <- extract.clade(tree, cycle[i])
  
  des <- Descendants(tree, cycle[i], type='all')
  
  nodes <- des[ which(des > length(tree$tip.label)) ] 
  
  tips <- des[ which(des < length(tree$tip.label)) ] #is equal to all tips descended from that node. So the node list is only useful for deleting clades belonging to other events.
  
  ### Check if any of the nodes are found in other events

  if(any(nodes %in% C_nodes_2)){
    x <- nodes[which(nodes %in% C_nodes_2)]
    for(y in x){
      desc <- unlist(Descendants(tree, y))
      tips <- tips[-which(tips %in% desc)]
    }
  }
  
  ### Check if any of the tips are found in other events
  
  if(any(tips %in% C_nodes_2)){
    tips <- tips[-which(tips %in% C_nodes_2) ]
  }
  
  #only get the tips with the correct state (D)
  tips <- tips[which(grepl( "C", all$states[tips]))]
  
  clade_rich<- c(clade_rich, length(tips)) #check that all tips have the same state included, or else there is an error
} #end loop, cycle through next node in D_list_1 (vector 'cycle')

#now clade_rich and tip_rich contain all richness derived from each independent colonization between the time period considered (here, 34-5.3 Ma)

C_rich <- sum(tip_rich + sum(clade_rich)) #richness of D derived from events occuring between 34-5.3 Ma

##### Some events don't actually lead to richness in the tree. Should subtract these nodes from sum and number counts.

cycle <- cycle[-which(clade_rich == 0)]
C_node_ages<- d[ which(d$C %in% cycle), 8 ] 
C_tip_ages <- d[ which(d$C %in% C_nodes_2[which(C_nodes_2<4572)]) , 8]
### Now we have all the information to fill in to 'data' for state D.

test[ 3,2 ]  <- C_rich
test[ 3,3 ] <- length(cycle)+tip_rich
test[ 3,4 ] <- sum(C_node_ages, C_tip_ages) 


################################################################################# state D

tip_rich <- length(which(D_nodes_2 < 4572)) #tip richness for B
clade_rich<- c()

## Now cycle thru the nodes

cycle <- D_nodes_2[which(D_nodes_2 >= 4572)]

for(i in 1:length(cycle)){
  clade <- extract.clade(tree, cycle[i])
  
  des <- Descendants(tree, cycle[i], type='all')
  
  nodes <- des[ which(des > length(tree$tip.label)) ] 
  
  tips <- des[ which(des < length(tree$tip.label)) ] #is equal to all tips descended from that node. So the node list is only useful for deleting clades belonging to other events.
  
  ### Check if any of the nodes are found in other events

  if(any(nodes %in% D_nodes_2)){
    x <- nodes[which(nodes %in% D_nodes_2)]
    for(y in x){
      desc <- unlist(Descendants(tree, y))
      tips <- tips[-which(tips %in% desc)]
    }
  }
  
  ### Check if any of the tips are found in other events

  if(any(tips %in% D_nodes_2)){
    tips <- tips[-which(tips %in% D_nodes_2) ]
  }
  
  #only get the tips with the correct state (D)
  tips <- tips[which(grepl( "D", all$states[tips]))]
  
  clade_rich<- c(clade_rich, length(tips)) #check that all tips have the same state included, or else there is an error
} #end loop, cycle through next node in D_list_1 (vector 'cycle')

#now clade_rich and tip_rich contain all richness derived from each independent colonization between the time period considered (here, 34-5.3 Ma)

D_rich <- sum(tip_rich + sum(clade_rich)) #richness of D derived from events occuring between 34-5.3 Ma

##### Some events don't actually lead to richness in the tree. Should subtract these nodes from sum and number counts.

cycle <- cycle[-which(clade_rich == 0)]
D_node_ages<- d[ which(d$D %in% cycle), 10 ] 
D_tip_ages <- d[ which(d$D %in% D_nodes_2[which(D_nodes_2<4572)]) , 10]
### Now we have all the information to fill in to 'data' for state D.

test[ 4,2 ]  <- D_rich
test[ 4,3 ] <- length(cycle)+tip_rich
test[ 4,4 ] <- sum(D_node_ages, D_tip_ages) 


################################################################################# state E

tip_rich <- length(which(E_nodes_2 < 4572)) #tip richness for B
clade_rich<- c()

## Now cycle thru the nodes

cycle <- E_nodes_2[which(E_nodes_2 >= 4572)]

for(i in 1:length(cycle)){
  clade <- extract.clade(tree, cycle[i])
  
  des <- Descendants(tree, cycle[i], type='all')
  
  nodes <- des[ which(des > length(tree$tip.label)) ] 
  
  tips <- des[ which(des < length(tree$tip.label)) ] #is equal to all tips descended from that node. So the node list is only useful for deleting clades belonging to other events.
  
  ### Check if any of the nodes are found in other events
 if(any(nodes %in% E_nodes_2)){
    x <- nodes[which(nodes %in% E_nodes_2)]
    for(y in x){
      desc <- unlist(Descendants(tree, y))
      tips <- tips[-which(tips %in% desc)]
    }
  }
  
  ### Check if any of the tips are found in other events

  if(any(tips %in% E_nodes_2)){
    tips <- tips[-which(tips %in% E_nodes_2) ]
  }
  
  #only get the tips with the correct state (D)
  tips <- tips[which(grepl( "E", all$states[tips]))]
  
  clade_rich<- c(clade_rich, length(tips)) #check that all tips have the same state included, or else there is an error
} #end loop, cycle through next node in D_list_1 (vector 'cycle')

#now clade_rich and tip_rich contain all richness derived from each independent colonization between the time period considered (here, 34-5.3 Ma)

E_rich <- sum(tip_rich + sum(clade_rich)) #richness of D derived from events occuring between 34-5.3 Ma

##### Some events don't actually lead to richness in the tree. Should subtract these nodes from sum and number counts.

cycle <- cycle[-which(clade_rich == 0)]
E_node_ages<- d[ which(d$E %in% cycle), 12 ] 
E_tip_ages <- d[ which(d$E %in% E_nodes_2[which(E_nodes_2<4572)]) , 12]
### Now we have all the information to fill in to 'data' for state D.

test[ 5,2 ]  <- E_rich
test[ 5,3 ] <- length(cycle)+tip_rich
test[ 5,4 ] <- sum(E_node_ages, E_tip_ages) 


################################################################################# state F

tip_rich <- length(which(F_nodes_2 < 4572)) #tip richness for B
clade_rich<- c()

## Now cycle thru the nodes

cycle <- F_nodes_2[which(F_nodes_2 >= 4572)]

for(i in 1:length(cycle)){
  clade <- extract.clade(tree, cycle[i])
  
  des <- Descendants(tree, cycle[i], type='all')
  
  nodes <- des[ which(des > length(tree$tip.label)) ] 
  
  tips <- des[ which(des < length(tree$tip.label)) ] #is equal to all tips descended from that node. So the node list is only useful for deleting clades belonging to other events.
  
  ### Check if any of the nodes are found in other events

  if(any(nodes %in% F_nodes_2)){
    x <- nodes[which(nodes %in% F_nodes_2)]
    for(y in x){
      desc <- unlist(Descendants(tree, y))
      tips <- tips[-which(tips %in% desc)]
    }
  }
  
  ### Check if any of the tips are found in other events
  
  if(any(tips %in% F_nodes_2)){
    tips <- tips[-which(tips %in% F_nodes_2) ]
  }
  
  #only get the tips with the correct state (D)
  tips <- tips[which(grepl( "F", all$states[tips]))]
  
  clade_rich<- c(clade_rich, length(tips)) #check that all tips have the same state included, or else there is an error
} #end loop, cycle through next node in D_list_1 (vector 'cycle')

#now clade_rich and tip_rich contain all richness derived from each independent colonization between the time period considered (here, 34-5.3 Ma)

F_rich <- sum(tip_rich + sum(clade_rich)) #richness of D derived from events occuring between 34-5.3 Ma

##### Some events don't actually lead to richness in the tree. Should subtract these nodes from sum and number counts.

cycle <- cycle[-which(clade_rich == 0)]
F_node_ages<- d[ which(d$F %in% cycle), 14 ] 
F_tip_ages <- d[ which(d$F %in% F_nodes_2[which(F_nodes_2<4572)]) , 14]
### Now we have all the information to fill in to 'data' for state D.

test[ 6,2 ]  <- F_rich
test[ 6,3 ] <- length(cycle)+tip_rich
test[ 6,4 ] <- sum(F_node_ages, F_tip_ages) 


################################################################################# state G

tip_rich <- length(which(G_nodes_2 < 4572)) #tip richness for B
clade_rich<- c()

## Now cycle thru the nodes

cycle <- G_nodes_2[which(G_nodes_2 >= 4572)]

for(i in 1:length(cycle)){
  clade <- extract.clade(tree, cycle[i])
  
  des <- Descendants(tree, cycle[i], type='all')
  
  nodes <- des[ which(des > length(tree$tip.label)) ] 
  
  tips <- des[ which(des < length(tree$tip.label)) ] #is equal to all tips descended from that node. So the node list is only useful for deleting clades belonging to other events.
  
  ### Check if any of the nodes are found in other events
  
  if(any(nodes %in% G_nodes_2)){
    x <- nodes[which(nodes %in% G_nodes_2)]
    for(y in x){
      desc <- unlist(Descendants(tree, y))
      tips <- tips[-which(tips %in% desc)]
    }
  }
  
  ### Check if any of the tips are found in other events
  
  if(any(tips %in% G_nodes_2)){
    tips <- tips[-which(tips %in% G_nodes_2) ]
  }
  
  #only get the tips with the correct state (D)
  tips <- tips[which(grepl( "G", all$states[tips]))]
  
  clade_rich<- c(clade_rich, length(tips)) #check that all tips have the same state included, or else there is an error
} #end loop, cycle through next node in D_list_1 (vector 'cycle')

#now clade_rich and tip_rich contain all richness derived from each independent colonization between the time period considered (here, 34-5.3 Ma)

G_rich <- sum(tip_rich + sum(clade_rich)) #richness of D derived from events occuring between 34-5.3 Ma

##### Some events don't actually lead to richness in the tree. Should subtract these nodes from sum and number counts.

cycle <- cycle[-which(clade_rich == 0)]
G_node_ages<- d[ which(d$G %in% cycle), 16 ] 
G_tip_ages <- d[ which(d$G %in% G_nodes_2[which(G_nodes_2<4572)]) , 16]
### Now we have all the information to fill in to 'data' for state D.

test[ 7,2 ]  <- G_rich
test[ 7,3 ] <- length(cycle)+tip_rich
test[ 7,4 ] <- sum(G_node_ages, G_tip_ages) 


################################################################################# state H

tip_rich <- length(which(H_nodes_2 < 4572)) #tip richness for B
clade_rich<- c()

## Now cycle thru the nodes

cycle <- H_nodes_2[which(H_nodes_2 >= 4572)]

for(i in 1:length(cycle)){
  clade <- extract.clade(tree, cycle[i])
  
  des <- Descendants(tree, cycle[i], type='all')
  
  nodes <- des[ which(des > length(tree$tip.label)) ] 
  
  tips <- des[ which(des < length(tree$tip.label)) ] #is equal to all tips descended from that node. So the node list is only useful for deleting clades belonging to other events.
  
  ### Check if any of the nodes are found in other events
  
  if(any(nodes %in% H_nodes_2)){
    x <- nodes[which(nodes %in% H_nodes_2)]
    for(y in x){
      desc <- unlist(Descendants(tree, y))
      tips <- tips[-which(tips %in% desc)]
    }
  }
  
  ### Check if any of the tips are found in other events

  if(any(tips %in% H_nodes_2)){
    tips <- tips[-which(tips %in% H_nodes_2) ]
  }
  
  #only get the tips with the correct state (D)
  tips <- tips[which(grepl( "H", all$states[tips]))]
  
  clade_rich<- c(clade_rich, length(tips)) #check that all tips have the same state included, or else there is an error
} #end loop, cycle through next node in D_list_1 (vector 'cycle')

#now clade_rich and tip_rich contain all richness derived from each independent colonization between the time period considered (here, 34-5.3 Ma)

H_rich <- sum(tip_rich + sum(clade_rich)) #richness of D derived from events occuring between 34-5.3 Ma

##### Some events don't actually lead to richness in the tree. Should subtract these nodes from sum and number counts.

cycle <- cycle[-which(clade_rich == 0)]
H_node_ages<- d[ which(d$H %in% cycle), 18 ] 
H_tip_ages <- d[ which(d$H %in% H_nodes_2[which(H_nodes_2<4572)]) , 18]
### Now we have all the information to fill in to 'data' for state D.

test[ 8,2 ]  <- H_rich
test[ 8,3 ] <- length(cycle)+tip_rich
test[ 8,4 ] <- sum(H_node_ages, H_tip_ages) 


################################################################################# state I

tip_rich <- length(which(I_nodes_2 < 4572)) #tip richness for B
clade_rich<- c()

## Now cycle thru the nodes

cycle <- I_nodes_2[which(I_nodes_2 >= 4572)]

for(i in 1:length(cycle)){
  clade <- extract.clade(tree, cycle[i])
  
  des <- Descendants(tree, cycle[i], type='all')
  
  nodes <- des[ which(des > length(tree$tip.label)) ] 
  
  tips <- des[ which(des < length(tree$tip.label)) ] #is equal to all tips descended from that node. So the node list is only useful for deleting clades belonging to other events.
  
  ### Check if any of the nodes are found in other events
  
  if(any(nodes %in% I_nodes_2)){
    x <- nodes[which(nodes %in% I_nodes_2)]
    for(y in x){
      desc <- unlist(Descendants(tree, y))
      tips <- tips[-which(tips %in% desc)]
    }
  }
  
  ### Check if any of the tips are found in other events
  
  if(any(tips %in% I_nodes_2)){
    tips <- tips[-which(tips %in% I_nodes_2) ]
  }
  
  #only get the tips with the correct state (D)
  tips <- tips[which(grepl( "I", all$states[tips]))]
  
  clade_rich<- c(clade_rich, length(tips)) #check that all tips have the same state included, or else there is an error
} #end loop, cycle through next node in D_list_1 (vector 'cycle')

#now clade_rich and tip_rich contain all richness derived from each independent colonization between the time period considered (here, 34-5.3 Ma)

I_rich <- sum(tip_rich + sum(clade_rich)) #richness of D derived from events occuring between 34-5.3 Ma

##### Some events don't actually lead to richness in the tree. Should subtract these nodes from sum and number counts.

cycle <- cycle[-which(clade_rich == 0)]
I_node_ages<- d[ which(d$I %in% cycle), 20 ] 
I_tip_ages <- d[ which(d$I %in% I_nodes_2[which(I_nodes_2<4572)]) , 20]
### Now we have all the information to fill in to 'data' for state D.

test[ 9,2 ]  <- I_rich
test[ 9,3 ] <- length(cycle)+tip_rich
test[ 9,4 ] <- sum(I_node_ages, I_tip_ages) 

##################### Now test is filled in for one map. Add to test_list 

test_list[[map]] <- test

}# end for one map. Move on to next map


####################################################################################################
####################################################################################################

####   Now can do downstream analyses on test_list for each of 100 maps

pvalues_sum <- as.data.frame(matrix(NA, 100, 4))
names(pvalues_sum) <- c("p_raw", "r2_raw", "p_log", "r2_log")

pvalues_numb <- as.data.frame(matrix(NA, 100, 4))
names(pvalues_numb) <- c("p_raw", "r2_raw", "p_log", "r2_log")


for(map in 1:100){
print(map)
  
test_list[[map]]$log_richness <- log10(test_list[[map]]$richness)

##### comparison 1: number of colonizations and richness

model1<- lm(richness~num_col, data=test_list[[map]]) #raw values of richness

model2<- lm(log_richness~num_col, data=test_list[[map]]) #log values of richness

pvalues_numb[map, 1] <- summary(model1)$coefficients[8]
pvalues_numb[map, 2] <- summary(model1)$r.squared
pvalues_numb[map, 3] <- summary(model2)$coefficients[8]
pvalues_numb[map, 4] <- summary(model2)$r.squared


##### comparison 2: summed age and richness

#store p-value and r-squared in data frame for all 50 maps

model1<- lm(richness~summed_age, data=test_list[[map]]) #raw values of richness

model2<- lm(log_richness~summed_age, data=test_list[[map]]) #log values of richness

pvalues_sum[map, 1] <- summary(model1)$coefficients[8]
pvalues_sum[map, 2] <- summary(model1)$r.squared
pvalues_sum[map, 3] <- summary(model2)$coefficients[8]
pvalues_sum[map, 4] <- summary(model2)$r.squared

}


setwd("~/Desktop/fish files may 2017/allperc_5-0_allareas")
write.csv(pvalues_sum, file="allperc_5-0_allareas_pvalues_summed.csv")
write.csv(pvalues_numb, file="allperc_5-0_allareas_pvalues_number.csv")

