### This is an example script for visualizing the colonization events during 5 Ma time bins

## This script is for STATE B (East Atlantic). Scripts for other regions are identical, and available by request.

# Input files are in the folders "stochastic_maps_independent_events" and "stochastic_maps_all_events". Users will need to set their directory appropriately.

setwd("~/Desktop/fish files may 2017/stochastic_results_sep15")
library(geiger)
library(phangorn)
tree <- read.tree("perc_tree_march2.tree") 


################################################################################# 
# number of bins if /5: 25 bins (0-125, every 5 Ma)

perc_rich <- as.data.frame(matrix(NA, 100, 25)) # a df of % of total richness for each time bin, for one map. rows= 1-100 maps. cols=1-24 time bins

num_col <- as.data.frame(matrix(NA, 100, 25)) # a df of the number of cols for each bin, for one map. same dims as above

for(map in 1:100){ # go through each of 100 maps
  print(map)
  
  setwd("~/Desktop/fish files may 2017/stochastic_results_sep15")
  
  d <- read.csv(paste0("independent_nodes+tips_stochastic_", map, ".csv"))
  
  all <- read.csv(paste0(map, "_all_states.csv"))
  
### IMPORTANT: for maps with fossil constraints, need to force colonizations along tips only to CIP to be 34 or less. This is because if a new colonization occured at a terminal taxa, we took half the stem age as the age of the colonization. However, some tips have a stem age >34 myr. 
### This section of code is only relevant for events to the CIP (state D) and not other regions.
### Turn this code off if not using fossil constraints. 
#for(i in 1:length(d$D)){
#  if(is.na(d$D_ages[i])){}else{
#    if(d$D_ages[i]>34){
#      d$D_ages[i]<-34
#    }}}


  clade_number <- c() # a vector for independent col events that leave descendants to B
  clade_rich<- c() # a vector for descendant richness for all col events to B
  
  cycle <- d[which(d$B >= 4572),]$B # all independent node-level col events to B
  
  for(i in 1:length(cycle)){
    clade <- extract.clade(tree, cycle[i])
    
    des <- Descendants(tree, cycle[i], type='all')
    
    nodes <- des[ which(des > length(tree$tip.label)) ] 
    
    tips <- des[ which(des <= length(tree$tip.label)) ] #is equal to all tips descended from that node. So the node list is only useful for deleting clades belonging to other events.
    
    ### Check if any of the nodes are found in other events
    if(any(nodes %in% cycle)){ #note that if there are no descendant nodes, nothing will happen
      x <- nodes[which(nodes %in% cycle)]
      for(y in x){  #cycle through each "secondary" event
        desc <- unlist(Descendants(tree, y)) #get the descendants of that node
        tips <- tips[-which(tips %in% desc)] #subtract those tips from the big list of tips
      }
    }
    # now, any tips left over in the big list should be direct descendants of that col event.
    # note that no tips might be left after this stage
    
    ### Check if any of the tips are found in other events
    
    if(any(tips %in% d[which(d$B <= 4571),]$B)){
      tips <- tips[-which(tips %in% d[which(d$B <= 4571),]$B) ]
    } # again if no tips are left, than nothing will happen
    
    #only get the tips with the correct state (B)
    tips <- tips[which(grepl( "B", all$states[tips]))]
    
    #if length(tips)=0, then the col event was just a stop-through for other events. Did not leave any descendants.
    
    clade_number <- c(clade_number, cycle[i])
    clade_rich<- c(clade_rich, length(tips)) 
    
  } #end loop, cycle through next node in vector 'cycle'
  
  ## Don't forget the independent tips!
  
  #now clade_rich and tip_rich contain all richness derived from each independent colonization between the time period considered (here, 34-5.3 Ma)
  
  ## delete those that are just "stop-gaps" (descendant richness = 0)
  
  delete <- which(clade_rich==0)
  clade_number <- clade_number[-delete]
  clade_rich <- clade_rich[-delete]
  
  ### Now add the independent tip events!
  
  clade_number <- c(clade_number, d[which(d$B <= 4571),]$B) # all independent tip-level col events to B
  clade_rich <- c(clade_rich, seq(from=1, to=1, length.out=length(d[which(d$B <= 4571),]$B)))
  
  ## get the corresponding ages
  
  clade_ages <- c()
  for(i in clade_number){
    clade_ages <- c(clade_ages, d$B_ages[ which(d$B %in% i) ] )
  }
  
  #now, add the map-specific data to the data frames for all maps.
  
  map_data <- as.data.frame(cbind(clade_number, clade_rich, clade_ages))
  
  total <- sum(map_data$clade_rich)
  x<- 0 #counter
  y<- 5 #counter
  z<- 1 #column counter
  for(i in 1:24){
    rows <- which((map_data$clade_ages > x) & (map_data$clade_ages <= y) )
    bin_data <- map_data[rows,]
    perc_rich[map, z] <- sum(bin_data$clade_rich)/total
    num_col[map, z] <- length(bin_data$clade_rich)
    x <- x+5
    y <- y+5
    z <- z+1
  } # next time bin
  
} # end loop through one map, move on to next map. 

colnames(perc_rich) <- as.character(c(5,	10,	15,	20,	25,	30,	35,	40,	45,	50,	55,	60,	65,	70,	75,	80,	85,	90,	95,	100,	105,	110,	115,	120,	125)) #upper bound of each time bin
colnames(num_col) <- as.character(c(5,	10,	15,	20,	25,	30,	35,	40,	45,	50,	55,	60,	65,	70,	75,	80,	85,	90,	95,	100,	105,	110,	115,	120,	125)) #upper bound of each time bin

setwd("~/Desktop/fish files may 2017/individual_cols/B")

write.csv(perc_rich, file= "B_perc_rich.csv" )
write.csv(num_col, file= "B_num_col.csv" )
