#### This script generates HiSSE files for binary comparisons, and performs HiSSE model comparison. 

Input files are included. Users will need to set their working directory accordingly.

library(geiger)

tree <- read.tree("perc_tree_march2.tree")
data <- read.csv("fish_areas_final_sep1.csv")
data<- data[ which(data$sampled.=="x"),] #tree data only

## Want to make trees only including tropical and marine species (tree for FW analysis contains all taxa)

fw_rows <- c() #exclusively FW taxa
x<-1
for(i in data$name_FishBase){
  if( (data$FW[x] ==1) & (data$WIP[x] ==0 & data$CP[x] ==0 & data$WA[x] == 0 & data$EA[x] == 0 & data$CIP[x] ==0 & data$EP[x] ==0 & data$NC[x] ==0 & data$SC[x] ==0)){
    fw_rows <- c(fw_rows, x)
  }
  x<- x+1
}

cold_rows <- c() #exclusively cold taxa, run this segment if aim is to make tree for warm water fish only
x<-1
for(i in data$name_FishBase){
  if( (data$NC[x] ==1| data$SC[x] ==1) & (data$WIP[x] ==0 & data$CP[x] ==0 & data$WA[x] == 0 & data$EA[x] == 0 & data$CIP[x] ==0 & data$EP[x] ==0 )){
    cold_rows <- c(cold_rows, x)
  }
  x<- x+1
}

## drop the tips in the tree corresponding to those rows.
#drop_rows <- c(fw_rows, cold_rows) # choose this if making tropical tree
drop_rows <- c(fw_rows) #choose this if making marine tree

drop_tips <- as.character(data[drop_rows,5]) #column "name_in_tree"
tree<-drop.tip(tree, drop_tips, trim.internal=TRUE)
#write.tree(tree, "perc_tropical_sep1.tree")
write.tree(tree, "perc_marine_sep1.tree")

################################# Now that we've made the trees, need to make the six data files.

########## Now, need to take out FW and cold rows in both tree and all species data, convert to binary, write file, and get a sampling fraction. 

## drop FW and cold rows in the tree data file.
#data<- data[ which(data$sampled.=="x"),] #tree data only

drop_rows <- c(fw_rows, cold_rows)
data <- data[-drop_rows ,]

### convert data (tropical inclusive, meaning anything occuring in the IWP counts as 1)

new_data <- vector()
for(i in 1:length(data$name_in_tree)){
  if(data$WIP[i]== 1 | data$CIP[i] == 1 | data$CP[i] ==1 ){
    new_data <- c(new_data, 1)
  }else{
    new_data <- c(new_data, 0)
  }
}
names(new_data) <- data$name_in_tree

write.csv(as.data.frame(new_data), file="tropical_inclusive_data_sep1.csv")

### OR ALTERNATIVELY, run the code below for tropical exclusive (meaning must be endemic to IWP to be 1)

new_data <- vector()
for(i in 1:length(data$name_in_tree)){
  if( (data$WIP[i]== 1 | data$CIP[i] == 1 | data$CP[i] ==1) & (data$EP[i] == 0 & data$WA[i] == 0 & data$EA[i] == 0) ){
    new_data <- c(new_data, 1)
  }else{
    new_data <- c(new_data, 0)
  }
}
names(new_data) <- data$name_in_tree

write.csv(as.data.frame(new_data), file="tropical_exclusive_data_sep1.csv")


## Now need to get the obis rows as well, to assess sampling fraction.

data <- read.csv("fish_areas_final_sep1.csv")

fw_rows <- c()
x<-1
for(i in data$name_FishBase){
  if( (data$FW[x] ==1) & (data$WIP[x] ==0 & data$CP[x] ==0 & data$WA[x] == 0 & data$EA[x] == 0 & data$CIP[x] ==0 & data$EP[x] ==0 & data$NC[x] ==0 & data$SC[x] ==0)){
    fw_rows <- c(fw_rows, x)
  }
  x<- x+1
}

cold_rows <- c()
x<-1
for(i in data$name_FishBase){
  if( (data$NC[x] ==1| data$SC[x] ==1) & (data$WIP[x] ==0 & data$CP[x] ==0 & data$WA[x] == 0 & data$EA[x] == 0 & data$CIP[x] ==0 & data$EP[x] ==0 )){
    cold_rows <- c(cold_rows, x)
  }
  x<- x+1
}

## drop FW and cold rows
drop_rows <- c(fw_rows, cold_rows)
data <- data[-drop_rows ,]

## now convert to binary

obis_data <- vector()
for(i in 1:length(data$name_FishBase)){
  if(data$WIP[i]== 1 | data$CIP[i] == 1 | data$CP[i] ==1 ){
    obis_data <- c(obis_data, 1)
  }else{
    obis_data <- c(obis_data, 0)
  }
}
names(obis_data) <- data$name_FishBase

### OR ALTERNATIVELY, run the code below for tropical exclusive (meaning must be endemic to IWP to be 1)

obis_data <- vector()
for(i in 1:length(data$name_FishBase)){
  if( (data$WIP[i]== 1 | data$CIP[i] == 1 | data$CP[i] ==1) & (data$EP[i] == 0 & data$WA[i] == 0 & data$EA[i] == 0) ){
    obis_data <- c(obis_data, 1)
  }else{
    obis_data <- c(obis_data, 0)
  }
}
names(obis_data) <- data$name_FishBase

### Get sampling fraction

f.0 <- length(which(new_data == 0)) / length(which(obis_data == 0))
f.1 <- length(which(new_data == 1)) / length(which(obis_data == 1))

f <- c(f.0, f.1)


#######################################################################

# Get data for marine inclusive + exclusive

data <- read.csv("fish_areas_final_sep1.csv")
data<- data[ which(data$sampled.=="x"),] #tree data only


fw_rows <- c() #exclusively FW taxa
x<-1
for(i in data$name_in_tree){
  if( (data$FW[x] ==1) & (data$WIP[x] ==0 & data$CP[x] ==0 & data$WA[x] == 0 & data$EA[x] == 0 & data$CIP[x] ==0 & data$EP[x] ==0 & data$NC[x] ==0 & data$SC[x] ==0)){
    fw_rows <- c(fw_rows, x)
  }
  x<- x+1
}


## drop FW and cold rows in the tree data file.
drop_rows <- c(fw_rows)
data <- data[-drop_rows ,]

### convert data (cold inclusive, meaning anything occuring in cold regions counts as 1)

new_data <- vector()
for(i in 1:length(data$name_in_tree)){
  if(data$NC[i]== 1 | data$SC[i] == 1 ){
    new_data <- c(new_data, 1)
  }else{
    new_data <- c(new_data, 0)
  }
}
names(new_data) <- data$name_in_tree

write.csv(as.data.frame(new_data), file="marine_inclusive_data_june7.csv")

### OR ALTERNATIVELY, run the code below for cold exclusive 

new_data <- vector()
for(i in 1:length(data$name_in_tree)){
  if( (data$NC[i] ==1| data$SC[i] ==1) & (data$WIP[i] ==0 & data$CP[i] ==0 & data$WA[i] == 0 & data$EA[i] == 0 & data$CIP[i] ==0 & data$EP[i] ==0 ) ){
    new_data <- c(new_data, 1)
  }else{
    new_data <- c(new_data, 0)
  }
}
names(new_data) <- data$name_in_tree

write.csv(as.data.frame(new_data), file="marine_exclusive_data_sep1.csv")


## Now need to get the obis rows as well, to assess sampling fraction.

data <- read.csv("fish_areas_final_sep1.csv")

fw_rows <- c()
x<-1
for(i in data$name_FishBase){
  if( (data$FW[x] ==1) & (data$WIP[x] ==0 & data$CP[x] ==0 & data$WA[x] == 0 & data$EA[x] == 0 & data$CIP[x] ==0 & data$EP[x] ==0 & data$NC[x] ==0 & data$SC[x] ==0)){
    fw_rows <- c(fw_rows, x)
  }
  x<- x+1
}


## drop FW rows
drop_rows <- c(fw_rows)
data <- data[-drop_rows ,]

## now convert to binary

obis_data <- vector()
for(i in 1:length(data$name_FishBase)){
  if(data$NC[i]== 1 | data$SC[i] == 1 ){
    obis_data <- c(obis_data, 1)
  }else{
    obis_data <- c(obis_data, 0)
  }
}
names(obis_data) <- data$name_FishBase

### OR ALTERNATIVELY, run the code below for tropical exclusive (meaning must be endemic to IWP to be 1)

obis_data <- vector()
for(i in 1:length(data$name_FishBase)){
  if( (data$NC[i] ==1| data$SC[i] ==1) & (data$WIP[i] ==0 & data$CP[i] ==0 & data$WA[i] == 0 & data$EA[i] == 0 & data$CIP[i] ==0 & data$EP[i] ==0 ) ){
    obis_data <- c(obis_data, 1)
  }else{
    obis_data <- c(obis_data, 0)
  }
}
names(obis_data) <- data$name_FishBase


f.0 <- length(which(new_data == 0)) / length(which(obis_data == 0))
f.1 <- length(which(new_data == 1)) / length(which(obis_data == 1))

f <- c(f.0, f.1)


#######################################################################

# Get data for freshwater inclusive + exclusive

data <- read.csv("fish_areas_final_sep1.csv")
data<- data[ which(data$sampled.=="x"),] #tree data only

### convert data (freshwater inclusive, meaning anything occuring in freshwater regions counts as 1)

new_data <- vector()
for(i in 1:length(data$name_in_tree)){
  if(data$FW[i]== 1 ){
    new_data <- c(new_data, 1)
  }else{
    new_data <- c(new_data, 0)
  }
}
names(new_data) <- data$name_in_tree

write.csv(as.data.frame(new_data), file="freshwater_inclusive_data_sep1.csv")

### OR ALTERNATIVELY, run the code below for freshwater exclusive 

new_data <- vector()
for(i in 1:length(data$name_in_tree)){
  if( (data$FW[i] ==1) & (data$NC[i] == 0 & data$SC[i] == 0 & data$WIP[i] ==0 & data$CP[i] ==0 & data$WA[i] == 0 & data$EA[i] == 0 & data$CIP[i] ==0 & data$EP[i] ==0 ) ){
    new_data <- c(new_data, 1)
  }else{
    new_data <- c(new_data, 0)
  }
}
names(new_data) <- data$name_in_tree

write.csv(as.data.frame(new_data), file="freshwater_exclusive_data_sep1.csv")


## Now need to get the obis rows as well, to assess sampling fraction.

data <- read.csv("fish_areas_final_sep1.csv")

## now convert to binary

obis_data <- vector()
for(i in 1:length(data$name_FishBase)){
  if(data$FW[i]== 1){
    obis_data <- c(obis_data, 1)
  }else{
    obis_data <- c(obis_data, 0)
  }
}
names(obis_data) <- data$name_FishBase

### OR ALTERNATIVELY, run the code below for freshwater exclusive

obis_data <- vector()
for(i in 1:length(data$name_FishBase)){
  if( (data$FW[i] ==1) & (data$NC[i] == 0 & data$SC[i] == 0 & data$WIP[i] ==0 & data$CP[i] ==0 & data$WA[i] == 0 & data$EA[i] == 0 & data$CIP[i] ==0 & data$EP[i] ==0 )  ){
    obis_data <- c(obis_data, 1)
  }else{
    obis_data <- c(obis_data, 0)
  }
}
names(obis_data) <- data$name_FishBase


f.0 <- length(which(new_data == 0)) / length(which(obis_data == 0))
f.1 <- length(which(new_data == 1)) / length(which(obis_data == 1))

f <- c(f.0, f.1)


###############################################################################
###############################################################################
###############################################################################

## Can actually run HiSSE models now
library(hisse)
tree <- read.tree("perc_tree_march2.tree")
data <- read.csv("freshwater_inclusive_data_sep1.csv")


## Function akaike weights is used later:
akaike.weights <- function(x)
{
  x <- x[!is.na(x)]
  delta.aic <- x - min(x, na.rm = TRUE)
  rel.LL <- exp(-0.5 * delta.aic)
  sum.LL <- sum(rel.LL, na.rm = TRUE)
  weights.aic <- rel.LL/sum.LL
  return(list(deltaAIC = delta.aic, rel.LL = rel.LL, weights = weights.aic))
}


f <- c( 0.2152912, 0.3388518) #input sampling fraction

## tropical inclusive:   0.3272951, 0.2008310
## tropical exclusive:  0.3377969, 0.1828477
## marine inclusive:  0.2006321, 0.3851429
## marine exclusive: 0.2376105, 0.2574899
## fw inclusive: 0.2152912, 0.3388518
## fw exclusive: 0.2395850, 0.3171947

trans.rates = TransMatMaker(hidden.states=TRUE)
trans.rates.nodual = ParDrop(trans.rates, c(3,5,8,10))

#Null Model One (equivalent to "Null Model Two") All div differences due to a hidden state and not an observed state

print("beginning model one")
Sys.time()

turnover.anc = c(1,1,2,2) #one rate for A, one rate for B
eps.anc = c(1,1,2,2) #one rate for A, one rate for B

#Allowing 8 transition rates

model_one = hisse(tree, data, f, hidden.states=TRUE, turnover.anc=turnover.anc, 
                  eps.anc=eps.anc, trans.rate=trans.rates.nodual, output.type="raw")

output_one = capture.output(model_one)
write(output_one, file="output_model_one_null2.txt")


#Null Model Two: All div differences due to an observed trait and not a hidden state (bisse equivalent)

print("beginning model two")
Sys.time()

turnover.anc = c(1,2,1,2) #one rate for 0, one rate for 1
eps.anc = c(1,2,1,2) #one rate for 0, one rate for 1

#need to change the trans matrix

trans.rates.bisse = TransMatMaker(hidden.states=FALSE)

model_two = hisse(tree, data, f, hidden.states=FALSE, turnover.anc=turnover.anc, 
                  eps.anc=eps.anc, trans.rate=trans.rates.bisse, output.type="raw")

output_two = capture.output(model_two)


write(output_two, file="output_model_two_nullBiSSE.txt")
Sys.time()

### Model THREE
trans.rates = TransMatMaker(hidden.states=TRUE)
trans.rates.nodual = ParDrop(trans.rates, c(3,5,8,10))

## Null 4 model
print("beginning model three") ### NOTE THAT THIS CODE REPLACED ON FEB 6 2017

model_three <- hisse.null4(tree, data, f, turnover.anc=rep(c(1,2,3,4),2), eps.anc=rep(c(1,2,3,4),2), output.type="raw") #this is the proper null4

output_three = capture.output(model_three)
write(output_three, file="output_model_three_null4.txt")

#Model Four: Full HiSSE Model with all rates varying, testing hypothesis that for 0 and 1, turnover, epsilon, and transitions vary

print("beginning model four full HiSSE")
Sys.time() #22:09

turnover.anc = c(1,2,3,4) #all rates vary
eps.anc = c(1,2,3,4) #all rates vary

#Allowing 8 transition rates

model_four = hisse(tree, data, f, hidden.states=TRUE, turnover.anc=turnover.anc, 
                   eps.anc=eps.anc, trans.rate=trans.rates.nodual, output.type="raw")

output_four = capture.output(model_four)
write(output_four, file="output_model_four_fullHiSSE.txt")


#Model Five: Epsilon constrained to be equal across 0 and 1

print("beginning model five")
Sys.time()

turnover.anc = c(1,2,3,4) #all rates vary
eps.anc = c(1,1,2,2) #one rate for all observed, one rate for all hidden
#Allowing 8 transition rates

model_five = hisse(tree, data, f, hidden.states=TRUE, turnover.anc=turnover.anc, 
                   eps.anc=eps.anc, trans.rate=trans.rates.nodual, output.type="raw")

output_five = capture.output(model_five)
write(output_five, file="output_model_five_eps.txt")


#Model Six: Turnover constrained to be equal across 0 and 1

print("beginning model six")
Sys.time()

turnover.anc = c(1,1,2,2) #one rate for all observed, one rate for all hidden
eps.anc = c(1,2,3,4) #all rates vary



#Allowing 8 transition rates

model_six = hisse(tree, data, f, hidden.states=TRUE, turnover.anc=turnover.anc, 
                  eps.anc=eps.anc, trans.rate=trans.rates.nodual, output.type="raw", sann=FALSE)

output_six = capture.output(model_six)
write(output_six, file="output_model_six_turn.txt")


AIC_list <- c(model_one$AIC, model_two$AIC, model_three$AIC, model_four$AIC, model_five$AIC, model_six$AIC)

write.csv(as.data.frame(akaike.weights(AIC_list)), file="akaike_weights_fresh_inclusive_sep1.csv")

###############################################
### Perform model-averaging, and get distribution of hidden states

# where mod= best-fitting hisse model

recon <- MarginRecon(mod$phy, mod$data, mod$f, pars=mod$solution, hidden.states=TRUE, n.cores=8)

#The code below is for one best-fit model. If more than one best-fit model, would instead input a list of two or more recon objects, then AIC.weights= the relative weights of each model

model.ave.rates <- GetModelAveRates(recon, AIC.weights = NULL, type="tips",
                                    bound.par.matrix=cbind(c(-2000,-2000,-2000,-2000,-2000),c(2000,2000,2000,130000,2000)) )

### Counting hidden state frequencies:

tips <- as.data.frame(recon$tip.mat) #for one reconstruction
tips$species <- recon$phy$tip.label

ntaxa <- length(tree$tip.label) #number of taxa in phylogeny
tips$maxprob <- NA
tips$state <- NA
for(i in 1:ntaxa){ 
  tips$maxprob[i] <- max(tips[i,2:5]) # edit column numbers depending on the number of hidden states (i.e. 2:9 for null-4 model)
  maxstate <- which(tips[i,2:5] == max(tips[i,2:5]))
  tips$state[i] <- colnames(tips)[maxstate+1]
}
head(tips)

counts <- count(tips, "state")
counts










